#lang racket
;;; exo13_1_sol.rkt pour le TP L3-INFO 2007-2008, Racket
;;; L'analyseur lexical pour la calculette interactive mfcalc

(require parser-tools/lex)

(define-lex-abbrevs
  (whitespace (union #\newline #\return #\tab #\space))
  (letter (union (char-range "a" "z") (char-range "A" "Z")))
  (digit (char-range "0" "9"))
  (int (repetition 1 +inf.0 digit))
  (int* (repetition 0 +inf.0 digit))
  (float (union (concatenation int "." int*)
                (concatenation "." int)))
  (function (union "log" "exp"))
  (boolean (union "true" "false"))
  (var (concatenation letter (repetition 0 +inf.0 (union letter digit)))))

(define-tokens value-tokens (NUM BOOL VAR FNCT))
(define-empty-tokens op-tokens (LPAR RPAR + - * / ^ = > < >= <= == != 
                                     EOF SEMICOLON NEG LOG EXP TRUE FALSE IF THEN ELSE LOWER_THAN_ELSE))

(printf "Generation du mfcalc-lexer... ")

(define get-lexeme
  (lexer
    ; la fin du fichier d'entree
    [(eof) 'EOF]
    ; on invoque recursivement le lexer pour sauter les espaces. La variable input-port est liee dans lexer
    [(repetition 1 +inf.0 whitespace) (get-lexeme input-port)]
    ; la variable lexeme est liee dans lexer et represente la chaine reconnue
    [(union "=" "+" "-" "*" "/" "^" "<" ">" "<=" ">=" "==" "!=" "if" "then" "else") (string->symbol lexeme)]
    [boolean (token-BOOL (string->symbol lexeme))]
    ["(" 'LPAR]
    [")" 'RPAR]
    [";" 'SEMICOLON]
    [function (token-FNCT (string->symbol lexeme))]                ; une fonction primitive - placee *avant* var
    [var (token-VAR (string->symbol lexeme))]                      ; une variable
    [(union int float) (token-NUM (string->number lexeme))]))      ; un entier ou un flottant est un nombre

(printf "ok. La fonction (get-lexeme port-in) est disponible.\n")

;;; Pour tester rapidement le lexer, on n'est pas oblige d'ouvrir un fichier-disque, il suffit
;;; d'une chaine de caracteres consideree comme un port d'entree dans lequel on peut lire. Notez
;;; qu'une chaine peut tenir en Scheme sur plusieurs lignes. 

(define string-test 
  "prix = 2*(
        100.5-1);
   if (x > 100.)==true then .1 else exp(log(x));")

(define (test1-lexer)
  (printf "\nTEST1 : analyse lexicale de la chaine:\n\"~a\"\n" string-test)
  (let ((p-in (open-input-string string-test)))                      ; un port d'entree associe a la chaine test
    (do ((i 0 (+ i 1)) (x (get-lexeme p-in) (get-lexeme p-in)))
      ((equal? x 'EOF) (list i 'lexemes 'ont 'ete 'lus))
      (printf "-------> ~a\n" x))))

(call-with-output-file "test-lex.dat"                               ; mise a jour du fichier test-lex5.dat
  (lambda (p-out)                                                    ; contenant la chaine string-test definie plus haut
    (fprintf p-out "~a" string-test))
  #:exists 'replace)

(define (test2-lexer)                               ; on lit cette fois la meme chaine mise dans un fichier
  (printf "\nTEST2 : analyse lexicale du fichier \"test-lex.dat\" :\n")
  (call-with-input-file "test-lex.dat"
    (lambda (p-in)
      (do ((i 0 (+ i 1)) (x (get-lexeme p-in) (get-lexeme p-in)))
      ((equal? x 'EOF) (list i 'lexemes 'ont 'ete 'lus))
      (printf "-------> ~a\n" x)))))

;(test1-lexer)
;(test2-lexer)

(define (exo3.1)    ; SOLUTION !!!!!!!!!!!!!!!!!
  (printf "Analyse lexicale du fichier test-lex.dat :\n")
  (call-with-input-file "test-lex.dat"
    (lambda (p-in)
      (let ((L '()))
        (do ((x (get-lexeme p-in) (get-lexeme p-in)))
          ((equal? x 'EOF) (reverse L))
          (printf "x = ~a\n" x)
          (when (and (token? x) (equal? (token-name x) 'NUM))
            (set! L (cons (token-value x) L))))))))

(exo3.1)



