;;; mfcalc-lexer-avec-@.scm pour le TP3 LINFO 2007-2008, DrScheme v371
;;; L'analyseur lexical pour la calculette interactive mfcalc avec la variable @

(if (not (equal? (version) "371"))
  (error "Installez la version 371 de DrScheme !!!"))

(require (lib "lex.ss" "parser-tools"))

(define-lex-abbrevs
  (whitespace (union #\newline #\return #\tab #\space))
  (letter (union (char-range "a" "z") (char-range "A" "Z")))
  (digit (char-range "0" "9"))
  (int (repetition 1 +inf.0 digit))
  (int* (repetition 0 +inf.0 digit))
  (float (union (concatenation int "." int*)
                (concatenation "." int)))
  (function (union "log" "exp"))
  (boolean (union "true" "false"))
  (var (union "@" (concatenation letter (repetition 0 +inf.0 (union letter digit))))))

(define-tokens value-tokens (NUM BOOL VAR FNCT))
(define-empty-tokens op-tokens (LPAR RPAR + - * / ^ = > < >= <= == != 
                                     EOF SEMICOLON NEG log exp true false if then else LOWER_THAN_ELSE))

(printf "Generation du mfcalc-lexer... ")

(define get-lexeme
  (lexer
    ; la fin du fichier d'entree
    [(eof) 'EOF]
    ; on invoque recursivement le lexer pour sauter les espaces. La variable input-port est liee dans lexer
    [(repetition 1 +inf.0 whitespace) (get-lexeme input-port)]
    ; la variable lexeme est liee dans lexer et represente la chaine reconnue
    [(union "=" "+" "-" "*" "/" "^" "<" ">" "<=" ">=" "==" "!=" "if" "then" "else") (string->symbol lexeme)]
    [boolean (token-BOOL (string->symbol lexeme))]
    ["(" 'LPAR]
    [")" 'RPAR]
    [";" 'SEMICOLON]
    [function (token-FNCT (string->symbol lexeme))]                ; une fonction primitive - placee *avant* var
    [var (token-VAR (string->symbol lexeme))]                      ; une variable
    [(union int float) (token-NUM (string->number lexeme))]))      ; un entier ou un flottant est un nombre

(printf "ok. La fonction (get-lexeme port-in) est disponible.\n")

;;; Pour tester rapidement le lexer, on n'est pas oblige d'ouvrir un fichier-disque, il suffit
;;; d'une chaine de caracteres consideree comme un port d'entree dans lequel on peut lire. Notez
;;; qu'une chaine peut tenir en Scheme sur plusieurs lignes. 

(define string-test 
  "prix = 2*(
        100.5-1);
   if (@ > 100.)==true then .1 else exp(log(@));")

(define (test1-lexer)
  (printf "\nTEST1 : analyse lexicale de la chaine:\n\"~a\"\n" string-test)
  (let ((p-in (open-input-string string-test)))                      ; un port d'entree associe a la chaine test
    (do ((i 0 (+ i 1)) (x (get-lexeme p-in) (get-lexeme p-in)))
      ((equal? x 'EOF) (list i 'lexemes 'ont 'ete 'lus))
      (printf "-------> ~a\n" x))))

(call-with-output-file "test-lex5.dat"                               ; mise a jour du fichier test-lex5.dat
  (lambda (p-out)                                                    ; contenant la chaine string-test definie plus haut
    (fprintf p-out "~a" string-test))
  'replace)

(define (test2-lexer)                               ; on lit cette fois la meme chaine mise dans un fichier
  (printf "\nTEST2 : analyse lexicale du fichier \"test-lex5.dat\" :\n")
  (call-with-input-file "test-lex5.dat"
    (lambda (p-in)
      (do ((i 0 (+ i 1)) (x (get-lexeme p-in) (get-lexeme p-in)))
      ((equal? x 'EOF) (list i 'lexemes 'ont 'ete 'lus))
      (printf "-------> ~a\n" x)))))

;(test1-lexer)
(test2-lexer)

"====================== TP exo 3.1 ===================="

(define (exo3.1)
  ...)

;(tester "(exo3.1)")


