#lang racket

;;; missp-sol.rkt, solution pour le TP L3-INFO, UNSA 2007-08, Racket 
;;; Utilisation de lex/yacc pour compiler du MISSP vers du Scheme...

;;; ------------------ construction du fichier de test ---------------------------------

(define string-for-file    ; contient un programme valide
  "foo {
         let b := false and n := 10 and id := proc(x) {x} and f := 9999 and iter := 9999
         in {letrec fac := proc(n,f) {if -n=0 then f(1) else fac(n-1,proc(r) {f(r*n)})}
              in {fac(n,id)}};
         n := 9;
         ++n;
         f := 1;
         f;
         iter := proc() { if n>0 then f else { f:=f*n; n--; iter()}};
         iter();
         print(f)
       } ")


(call-with-output-file "test.msa"        ; mise a jour du fichier de test
    (lambda (p-out)
      (fprintf p-out "~a\n" string-for-file))
    #:exists 'replace)

;;; ------------------------------------------------------------------------------------

(require parser-tools/lex parser-tools/yacc)

(printf "Generating lexer...~n")

(define-lex-abbrevs
  [whitespace (union #\newline #\return #\tab #\space)]
  [letter (union (char-range "a" "z") (char-range "A" "Z"))]
  [digit (char-range "0" "9")]
  [int (repetition 1 +inf.0 digit)]
  [int* (repetition 0 +inf.0 digit)]
  [float (union (concatenation int "." int*) (concatenation "." int))]
  [boolean (union "true" "false")]
  [var (concatenation letter (repetition 0 +inf.0 (union letter digit)))])

(define-tokens value-tokens (BOOL NUM VAR))

(define-empty-tokens op-tokens
  (NEWLINE LPAR RPAR LBRA RBRA COLON SEMICOLON EOF define if then else LOWER_THAN_ELSE true false
           begin end let letrec in and NEG proc + - * / ^ = > < >= <= := -- ++))

(define get-lexeme
  (lexer
   [(eof) 'EOF]
   [(repetition 1 +inf.0 whitespace) (get-lexeme input-port)]
   ["(" 'LPAR]
   [")" 'RPAR]
   ["{" 'LBRA]
   ["}" 'RBRA]
   ["," 'COLON]
   [";" 'SEMICOLON]
   ;[#\newline 'newline]
   [boolean (token-BOOL (string=? "true" lexeme))]   ; on transforme true et false en #t et #f
   [(union "if" "then" "else" "let" "letrec" "define" "proc" "begin" "end" "in" "and") (string->symbol lexeme)]
   [(union "+" "-" "*" "/" "^" ">" "<" ">=" "<=" "=" ":=" "--" "++") (string->symbol lexeme)]
   [var (token-VAR (string->symbol lexeme))]
   [int (token-NUM (string->number lexeme))]
   [float (token-NUM (string->number lexeme))]
   [(char-range #\000 #\377) (error "Unparseable" (string->symbol lexeme))]))

(when #f             ; mettre #t pour activer ce test !!
  (printf "Testing lexer on file \"test.msa\"...\n")
  (call-with-input-file "test.msa"
    (lambda (p-in) 
      (do ((i 0 (+ i 1)) (x (get-lexeme p-in) (get-lexeme p-in)))
        ((equal? x 'EOF) 'fini)
        (printf "~a : ~a~n" i x)))))

; (error "STOP AFTER LEXER")

(printf "Generating compiler...\n")

(define parse
  (parser
   [start go]
   ;[debug "debug-compiler-tp.txt"]
   [end EOF]
   
   [tokens value-tokens op-tokens]
   
   [error (lambda (a b c) (printf "ERROR with token : ~a\n" b))]  ; ?
   
   [precs (nonassoc LOWER_THAN_ELSE)
          (nonassoc else)
          (nonassoc :=)
          (nonassoc = < <= > >=)
          (left + -)
          (left * /)
          (nonassoc NEG)
          (right ^)
          (nonassoc -- ++)]
   
   [grammar
    (go       [() 'fini]
              [(program) $1])
    (program  [(VAR bloc) `(define (,$1) ,$2)]                 ; définition de fonction en MISSP
              [(bloc) `(define (,(gensym 'prgm)) ,$1)])        ; suite d'instructions
    (bloc     [(LBRA exp+ RBRA) `(begin ,@$2)])
    (exp+     [(exp) `(,$1)]
              [(exp+ SEMICOLON exp) `(,@$1 ,$3)])              ; Yacc prefere les recursivites gauches !
    (exp      [(NUM) $1]
              [(BOOL) $1]
              [(VAR) $1]
              [(VAR := exp) `(set! ,$1 ,$3)]
              [(if exp then exp else exp) `(if ,$2 ,$4 ,$6)]
              [(if exp then exp) (prec LOWER_THAN_ELSE) `(if ,$2 ,$4)]
              [(bloc) $1]
              [(let decls in bloc) `(let ,$2 ,$4)]
              [(letrec decls in bloc) `(letrec ,$2 ,$4)]
              [(VAR LPAR oprands RPAR) `(,$1 ,@$3)]
              [(proc LPAR varlist RPAR bloc) `(lambda ,$3 ,$5)]
              [(- exp) (prec NEG) `(- 0 ,$2)]
              [(LPAR exp RPAR) $2]
              [(exp + exp) `(+ ,$1 ,$3)]
              [(exp * exp) `(* ,$1 ,$3)]
              [(exp - exp) `(- ,$1 ,$3)]
              [(exp / exp) `(/ ,$1 ,$3)]
              [(exp ^ exp) `(expt ,$1 ,$3)]
              [(exp < exp) `(< ,$1 ,$3)]
              [(exp > exp) `(> ,$1 ,$3)]
              [(exp <= exp) `(<= ,$1 ,$3)]
              [(exp >= exp) `(>= ,$1 ,$3)]
              [(exp = exp) `(= ,$1 ,$3)]
              [(VAR --) `(let ((x ,$1)) (set! ,$1 (- ,$1 1)) x)] 
              [(VAR ++) `(let ((x ,$1)) (set! ,$1 (+ ,$1 1)) x)] 
              [(-- VAR) `(begin (set! ,$2 (- ,$2 1)) ,$2)]
              [(++ VAR) `(begin (set! ,$2 (+ ,$2 1)) ,$2)])
    (varlist  [() '()]
              [(VAR suite-varlist) `(,$1 ,@$2)])
    (suite-varlist [() '()]
                   [(suite-varlist COLON VAR) `(,@$1 ,$3)])   ; notez les recursivites gauches !!!
    (oprands  [() '()]
              [(exp suite-oprands) `(,$1 ,@$2)])
    (suite-oprands [() '()]
                   [(suite-oprands COLON exp) `(,@$1 ,$3)])   ; notez les recursivites gauches !!!
    (decls    [() '()]
              [(VAR := exp suite-decls) `((,$1 ,$3) ,@$4)])
    (suite-decls [() '()]
                 [(suite-decls and VAR := exp) `(,@$1 (,$3 ,$5))])]))   ; notez les recursivites gauches !!!


(printf "--------------------------------------------------\n\n")

(define (missp->scheme s)     ; le TRADUCTEUR : string-missp -> pretty-string-scheme
  (let* ((p-in (open-input-string s))
         (scan (lambda () (get-lexeme p-in)))
         (string-port-out (open-output-string)))
    (pretty-print (parse scan) string-port-out)
    (get-output-string string-port-out)))

(define essai "foo {let x:=2 in {print(x-3); print(x)}}")    ; un p'tit essai ?...
(printf "Je traduis du texte MISSP : ~s\n" essai)
(printf "en texte Scheme :\n")
(define resultat (missp->scheme essai))
(printf "~s\n\n" resultat)

(printf "--------------------------------------------------\n\n")
       
(define (test-file)
  (printf "Un autre test : je traduis le fichier \"test.msa\"...\n")
  (call-with-input-file "test.msa"                            ; compilation du fichier de test
    (lambda (p-in)
      (let ((scan (lambda () (get-lexeme p-in))))             ; un scanneur sur le port p-in
        (letrec ((one-bloc (lambda ()
                             (let ((prgm (parse scan)))       ; on compile une expression
                               (if (equal? prgm 'fini)
                                   'fini
                                   (begin (pretty-print prgm) (one-bloc)))))))
          (one-bloc))))))       

(test-file)

(define (test-interactive)
  (printf "Test interactif ! Entrez un programme valide puis appuyez sur EOF\n")
  (let ((scan (lambda () (get-lexeme (current-input-port)))))
    (letrec ((one-bloc (lambda ()
                         (printf "? ")
                         (let ((prgm (parse scan)))
                           (if (equal? prgm 'fini)
                               'fini
                               (begin (pretty-print prgm) (one-bloc)))))))
      (one-bloc))))

;(test-interactive)

